/*	Copyright (C) 2003-2013 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "network.pmh"

namespace fe
{

SignalReceiver::SignalReceiver(sp<Scope> spScope, sp<ChunkSpool> spChunkSpool)
{
	m_spScope = spScope;
	m_spChunkSpool = spChunkSpool;
	m_spStream = new data::BinaryStream(m_spScope);
}

SignalReceiver::~SignalReceiver(void)
{
}

void SignalReceiver::bind(sp<SignalerI> spSignalerI, sp<Layout> spLayout)
{
	m_spSignalerI = spSignalerI;
}

void SignalReceiver::handle(Record &signal)
{
	sp<Chunk> spChunk = m_spChunkSpool->poll();

	sp<RecordGroup>	spRG;

	while(spChunk.isValid())
	{
		std::istringstream ist(spChunk->string());

		spRG = m_spStream->input(ist);

		for(RecordGroup::iterator it = spRG->begin(); it != spRG->end(); it++)
		{
			sp<RecordArray> spRA(*it);
			for(int i = 0; i < spRA->length(); i++)
			{
				Record record = spRA->getRecord(i);
				m_spSignalerI->signal(record);
			}
		}

		spChunk = m_spChunkSpool->poll();
	}
}

} /* namespace */

