/*	Copyright (C) 2003-2013 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "signal/signal.pmh"	// direct access to ChainSignaler

#include "network/network.h"

class AHandler : virtual public fe::HandlerI, virtual public fe::BindI
{
	public:
		AHandler(void){}
virtual	~AHandler(void){}
virtual	void	handle(fe::Record &signal)
		{
			feLogGroup("network","a: %d\n", a(signal));
			feLogGroup("network","ttl: %d\n", ttl(signal));
		}
virtual	void	bind(fe::sp<fe::SignalerI> spSignalerI,
						fe::sp<fe::Layout> spLayout)
		{
			a.setup(spLayout->scope(), "a");
			ttl.setup(spLayout->scope(), FE_USE(":TTL"));
		}
	private:
		fe::Accessor<int>	a;
		fe::Accessor<int>	ttl;
};

int main(int argc, char *argv[])
{
	fe::UnitTest unitTest;

	U32 complete=FALSE;

	try
	{
		if(argc<2)
			feX("Not enough arguments");

		fe::Socket::startup();
		fe::sp<fe::Master> spMaster(new fe::Master);
		fe::sp<fe::Registry> spRegistry=spMaster->registry();

		fe::Result result=spRegistry->manage("fexNetworkDL");
		if(fe::failure(result))
		{
			feX(argv[0], "could not manage fexNetworkDL %p", result);
		}

		fe::sp<fe::Scope> spScope(new fe::Scope(*spMaster.raw()));
		fe::sp<fe::SignalerI> spSignaler(new fe::ChainSignaler());
		spSignaler->setName("SignalerI");

		fe::Accessor<int>	a(spScope,"a");
		fe::Accessor<int>	ttl(spScope, FE_USE(":TTL"));
		fe::sp<fe::Layout>	spSignal = spScope->declare("SIGNAL");
		spSignal->populate(a);
		spSignal->populate(ttl);

		fe::sp<fe::Layout>	spHB = spScope->declare("HEARTBEAT");
		fe::Record hb = spScope->createRecord(spHB);

		fe::sp<fe::HandlerI> spHandlerI(new AHandler);
		spHandlerI->setName("HandlerI");
		spSignaler->insert(spHandlerI, spSignal);

		fe::sp<fe::ServerI>	spServer(spRegistry->create("ServerI"));
		if(!spServer.isValid())
		{
			feX(argv[0], "could not create Server component");
		}
		spServer->addLayout(spSignal);

		spServer->start(spSignaler, (short)atol(argv[1]), spHB);
		for(int i = 0; i < 60; i++)
		{
			spSignaler->signal(hb);
			fe::milliSleep(100);
		}
		spServer->stop();

		complete=TRUE;
//		spScope->shutdown();
	}
	catch(fe::Exception &e)
	{
		e.log();
	}

	unitTest(complete,"incomplete");
	return unitTest.failures();
	return 0;
}

