if __name__ == '__main__':
	import forge
else:
	import ctypes

	class Context:
		def __init__(self, name = "Context"):
#			print "Context.init " + name
			self.objName = name

			terminalDL = ctypes.CDLL("libfexTerminalDL.so")
			self.fe_terminal_execute = terminalDL.fe_terminal_execute

		def execute(self, command):
			size = 256
			result = ctypes.create_string_buffer(size)

			print "\n[36m" + command + "[0m"
			self.fe_terminal_execute(size,result,command)
			print "[34m" + result.value + "[0m"

		def name(self):
			return self.objName

		def manage(self, library):
#			print "Context.manage " + library

			command = "manage " + library
			self.execute(command)

		def load(self, path, name = False):
			name = name or path

#			print "Context.load " + path + " name " + name

			command = "load " + name + " " + path
			self.execute(command)

			surface = Surface(self, name)
			return surface

		def create(self, operator, name = False):
			name = name or operator

#			print "Context.create " + operator + " name " + name

			command = "create " + operator + " " + name
			self.execute(command)

			operator = Operator(self, name)
			return operator

	class Operator:
		def __init__(self, context, name = "Operator"):
#			print "Operator.init " + name
			self.context = context
			self.objName = name

		def __del__(self):
#			print "Operator.del " + self.objName
			self.release()

		def name(self):
			return self.objName

		def operate(self, input0 = None, input1 = None, input2 = None, input3 = None, name = False):
			name = name or (self.objName + ".out")

#			print "Operator.operate " + name

			command = "signal " + self.objName + " " + name

			if input0:
#				print "  input0 " + input0.name()
				command += " " + input0.name()
			if input1:
#				print "  input1 " + input1.name()
				command += " " + input1.name()
			if input2:
#				print "  input2 " + input2.name()
				command += " " + input2.name()
			if input3:
#				print "  input3 " + input3.name()
				command += " " + input3.name()

			self.context.execute(command)

			surface = Surface(self.context, name)
			return surface

		def set(self, attribute, value, typename = False):
			if not typename:
				if type(value) is float:
					typename = "real"
				elif type(value) is int:
					typename = "integer"
				elif type(value) is bool:
					typename = "boolean"
				elif type(value) is str:
					typename = "string"
				else:
					print "Operator.set incompatible type " + str(type(value))
					print type(value)
					return

#			print "Operator.set \"" + attribute + "\" " + typename + " " + str(value)

			command = "set %s %s %s %s" % (self.objName, attribute, typename, str(value))
			self.context.execute(command)

		def get(self, attribute, typename = ""):
			print "Operator.get " + attribute + " " + typename

			# TODO

		def release(self):
#			print "Operator.release"

			command = "release " + self.objName
			self.context.execute(command)

	class Surface:
		def __init__(self, context, name = "Surface"):
#			print "Surface.init " + name
			self.context = context
			self.objName = name

		def __del__(self):
#			print "Surface.del " + self.objName
			self.release()

		def name(self):
			return self.objName

		def save(self, path):
#			print "Surface.save " + path

			command = "save %s %s" % (self.objName, path)
			self.context.execute(command)

		def release(self):
#			print "Surface.release"

			command = "release " + self.objName
			self.context.execute(command)
