import os
import sys
import re
forge = sys.modules["forge"]

def find_vdb_so():
	libFiles = os.listdir(forge.openvdb_lib)
	for libFile in libFiles:
		if re.match(".*\.so", libFile):
			return libFile

	return "libopenvdb.so.1.1"

def find_houdini_vdb():
	houdini_source_index = len(forge.houdini_sources)-1
	while houdini_source_index >= 0:
		houdini_source = forge.houdini_sources[houdini_source_index]
		houdini_source_version = None

		tokens = houdini_source[0].split(".")
		if tokens > 1:
			houdini_source_version = float(tokens[0]+"."+tokens[1])
		elif is_int(houdini_source[0]):
			houdini_source_version = int(houdini_source[0])

		# avoid version with Houdini 14.0 until it gets ironed out
		if houdini_source_version and houdini_source_version<14:
			return houdini_source

		houdini_source_index -= 1

	return None

def setup(module):

	srcList = [	"SurfaceVDB",
				"vdb.pmh",
				"vdbDL" ]

	dll = module.DLL( "fexVdbDL", srcList )

	deplibs = forge.corelibs+ [
				"fexSurfaceDLLib" ]

	forge.deps( ["fexVdbDLLib"], deplibs )

#	module.includemap = { 'vdb' : module.modPath }
	module.includemap = { 'vdb' : os.path.join(forge.openvdb_include, '..') }
	default_openexr = forge.openexr_source + '/include'
	if os.path.exists(default_openexr):
		module.includemap['openexr'] = default_openexr

	# HACK needs to be configurable
	module.linkmap = {}

	# HACK use boost and openvdb from last listed houdini version
	houdini_source = find_houdini_vdb()
	if houdini_source:
		module.includemap['houdini_boost'] = houdini_source[1]
		module.linkmap['houdini_boost'] = '-L' + houdini_source[2]

#		module.linkmap['vdb'] = ' -lopenvdb'
#	else:

	soFile = find_vdb_so()

	module.linkmap['vdb'] = "-Xlinker -rpath='" + forge.openvdb_lib + "'"
	module.linkmap['vdb'] += ' -L' + forge.openvdb_lib + ' -l:' + soFile

	forge.tests += [
		("inspect.exe",		"fexVdbDL",				None,		None) ]

	for line in open(forge.openvdb_include + "/version.h").readlines():
		if re.match("#define OPENVDB_VERSION_NAME.*", line):
			version = line.split(' ')[2][1:-1]
			sys.stdout.write(version + " ")
			break

#	module.Module('openvdb')

def auto(module):
	if forge.openvdb_include == '':
		return 'unset'

	test_file = """
#include "openvdb/openvdb.h"

int main(void)
{
	return(0);
}
    \n"""

	houdini_source = find_houdini_vdb()
	if houdini_source:
		forge.includemap['houdini_boost'] = houdini_source[1]
		forge.linkmap['houdini_boost'] = '-L' + houdini_source[2]

	forge.includemap['vdb'] = os.path.join(forge.openvdb_include, '..')
	default_openexr = forge.openexr_source + '/include'
	if os.path.exists(default_openexr):
		forge.includemap['openexr'] = default_openexr

	soFile = find_vdb_so()

	forge.linkmap['vdb'] = "-Xlinker -rpath='" + forge.openvdb_lib + "'"
	forge.linkmap['vdb'] += ' -L' + forge.openvdb_lib + ' -l:' + soFile

	result = forge.cctest(test_file)

	forge.includemap['houdini_boost'] = ''
	forge.linkmap['houdini_boost'] = ''
	forge.includemap['openexr'] = ''
	forge.includemap['vdb'] = ''
	forge.linkmap['vdb'] = ''

	return result
